proc powDebugDataPrint { title string } {
     puts "$title"
     set k 0
     for {set i 0} {$i < [string length $string]} {incr i 80} {
        set currentStr [string range $string $i [expr $i + 79]]
        puts "<$currentStr>"
        incr k
     }
     puts "count: $k"
}

proc assembleWcsLabel { filehandle img selection } {
    global powWCSLabel

    set target $selection
    if { $selection == "DEFAULT" } {
       set selection ""
    }
    set powWCSLabel(xlabel,$img,$target) ""
    set powWCSLabel(ylabel,$img,$target) ""
    set powWCSLabel(xunit,$img,$target) "pixels"
    set powWCSLabel(yunit,$img,$target) "pixels"

    set x_label ""
    set y_label ""
    set x_unit "pixels"
    set y_unit "pixels"
    if { ![catch {set tmp [getKeyword $filehandle CTYPE1$selection]}] } {
       set v [lindex [lindex $tmp 0] 1]
       set x_label [string trim $v {' }]
       set powWCSLabel(xlabel,$img,$target) $x_label
    }
    if { ![catch {set tmp [getKeyword $filehandle CTYPE2$selection]}] } {
       set v [lindex [lindex $tmp 0] 1]
       set y_label [string trim $v {' }]
       set powWCSLabel(ylabel,$img,$target) $y_label
    }
    if { ![catch {set tmp [getKeyword $filehandle CUNIT1$selection]}] } {
       set v [lindex [lindex $tmp 0] 1]
       set x_unit [string trim $v {' }]
       set powWCSLabel(xunit,$img,$target) $x_unit
    }
    if { ![catch {set tmp [getKeyword $filehandle CUNIT2$selection]}] } {
       set v [lindex [lindex $tmp 0] 1]
       set y_unit [string trim $v {' }]
       set powWCSLabel(yunit,$img,$target) $y_unit
    }

    set z_label "counts"
    if { ![catch {set tmp [getKeyword $filehandle BUNIT$selection]}] } {
       set v [lindex [lindex $tmp 0] 1]
       set z_label [string trim $v {' }]
    }
    set powWCSLabel(zlabel,$img,$target) $z_label
}

proc getWcs {{dest {}} {RAColNum_ {}} {DecColNum_ {}} } {
    global filehandle
    if { $RAColNum_ == "" || $DecColNum_ == "" } {
       if { [catch {set wcs [$filehandle get wcs -m $dest]}] } {
          return ""
       } else {
          return $wcs
       }
    } else {
        set wcs [$filehandle get wcs -m $dest $RAColNum_ $DecColNum_]
        return $wcs
    }

}

proc rotationSend { str img } {
   global powHeaderWcsKeyWord powWCSInfo powWCSToken powWCS

   set rotationRate 80
   set i 0
   while { 1 } {
      set currentStr [string range $str $i [expr $i + [expr $rotationRate - 1]]]
      incr i $rotationRate
      if { [string trim $currentStr] == "" } {
         if { $i > [string length $str] } break
         continue
      }
      ::powCmds::wcsHeader $img $rotationRate $currentStr continue
   }
}

proc getHeaderKeyWord { str img } {
   global powHeaderWcsKeyWord powWCSInfo powWCSToken powWCS

   if [info exists powHeaderWcsKeyWord($img,DEFAULT)] {

      foreach letter [list a b c d e f g h i j k l m n o p q r s t u v w x y z] {
         if [info exists powHeaderWcsKeyWord($img,$letter)] {
            unset powHeaderWcsKeyWord($img,$letter)
         }
      }
      if [info exists powHeaderWcsKeyWord($img,NONE)] {
         unset powHeaderWcsKeyWord($img,NONE)
      }
      if [info exists powHeaderWcsKeyWord($img,END)] {
         unset powHeaderWcsKeyWord($img,END)
      }
      if [info exists powHeaderWcsKeyWord($img,DEFAULT)] {
         unset powHeaderWcsKeyWord($img,DEFAULT)
      }
      unset powHeaderWcsKeyWord
   }

   set i 0
   set numCard 0
   set numCoord 0
   set numCardInCoord 0
   set powWCSToken($img) { DEFAULT }
   while { 1 } {
      set currentStr [string range $str $i [expr $i + 79]]
      incr i 80
      if { [string trim $currentStr] == "" } {
         if { $i > [string length $str] } break
         continue
      }
      set currentStrToken [split $currentStr "="]
      set header [string trim [lindex $currentStrToken 0]]
      incr numCard

      if { [llength $currentStrToken] == 2 } {

         switch -regexp -- $header {
             {CTYPE[0-9][A-Z]?} -
             {CUNIT[0-9][A-Z]?} -
             {CRVAL[0-9][A-Z]?} -
             {CRPIX[0-9][A-Z]?} -
             {CD[0-9][_][0-9][A-Z]?} -
             {CDELT[0-9][A-Z]?} -
             {CROTA[0-9][A-Z]?} -
             {TCTYP[0-9][A-Z]?} -
             {TCUNI[0-9][A-Z]?} -
             {TCRVL[0-9][A-Z]?} -
             {TCRPX[0-9][A-Z]?} -
             {TCDLT[0-9][A-Z]?} -
             {TCD[0-9][A-Z]?} -
             {TCROT[0-9][A-Z]?} -
             {OFFSET[0-9][A-Z]?} {
                incr numCardInCoord
                set lastChar [string toupper [string range $header end end]]
                regsub {[A-Z]} $lastChar {} testChar
                if { $testChar == "" } {
                   set headerLength [string length [lindex $currentStrToken 0]]
                   set newHeader [string range $header 0 [expr [string length $header] - 2]]
                   set newStr [format "%-${headerLength}s=%s" $newHeader \
                                                                [lindex $currentStrToken 1]]
                   if { ![info exists powHeaderWcsKeyWord] || \
                        ![info exists powHeaderWcsKeyWord($img,$lastChar)] } {
                      incr numCoord
                      set powHeaderWcsKeyWord($img,$lastChar) $newStr
                      set powWCSInfo($img,$lastChar) [getWcs $lastChar]
                      lappend powWCSToken($img) $lastChar
                   } else {
                      set powHeaderWcsKeyWord($img,$lastChar) \
                          [format "%s%s" $powHeaderWcsKeyWord($img,$lastChar) $newStr]
                   }
                } else {
                   if { ![info exists powHeaderWcsKeyWord] || \
                        ![info exists powHeaderWcsKeyWord($img,DEFAULT)] } {
                      incr numCoord
                      set powHeaderWcsKeyWord($img,DEFAULT) $currentStr
                   } else {
                      set powHeaderWcsKeyWord($img,DEFAULT) \
                          [format "%s%s" $powHeaderWcsKeyWord($img,DEFAULT) $currentStr]
                   }
                }
             }
             default {
                if { ![info exists powHeaderWcsKeyWord] || \
                     ![info exists powHeaderWcsKeyWord($img,NONE)] } {
                   set powHeaderWcsKeyWord($img,NONE) $currentStr
                } else {
                   set powHeaderWcsKeyWord($img,NONE) \
                       [format "%s%s" $powHeaderWcsKeyWord($img,NONE) $currentStr]
                }
             }
         }
      } else {
         switch -glob -- $header {
             "END*" {
                set powHeaderWcsKeyWord($img,END) $currentStr
             }
             default {
                if { ![info exists powHeaderWcsKeyWord] || \
                     ![info exists powHeaderWcsKeyWord($img,NONE)] } {
                   set powHeaderWcsKeyWord($img,NONE) $currentStr
                } else {
                   set powHeaderWcsKeyWord($img,NONE) \
                       [format "%s%s" $powHeaderWcsKeyWord($img,NONE) $currentStr]
                }
             }
         }
      }
      if { $i > [string length $str] } break
   }

   if { $numCoord > 0 } {
      set numCardPerCoord [expr $numCardInCoord / $numCoord]

      return [list $numCard $numCard]
   } else {
      return [list 0 $numCard]
   }
}

proc assembleWcsHeader { img {selection "DEFAULT"} } {
   global powHeaderWcsKeyWord

   # regular header
   if { $selection == "NOWCS" } {
      return [format "%s%s" $powHeaderWcsKeyWord($img,NONE) \
                            $powHeaderWcsKeyWord($img,END)]
   } else {
      return [format "%s%s%s" $powHeaderWcsKeyWord($img,NONE) \
                              $powHeaderWcsKeyWord($img,$selection) \
                              $powHeaderWcsKeyWord($img,END)]
   }
}

proc getKeyword {filehandle keyword} {
    # get $keyword
    $filehandle get keyword ^$keyword\$
}

set helpMsg "Usage: POWplotServer  ?-display client? ?fitsimage?
 client: XPA entry point of form IP_Address:portNumber, pointing to
         a remote POW session to which plot commands should be sent"

global env

set ppFitsDataType(8) 0
set ppFitsDataType(16) 1
set ppFitsDataType(32) 2
set ppFitsDataType(-32) 3
set ppFitsDataType(-64) 4


#
# Load libraries
#

puts "POW_DISPLAY $env(POW_DISPLAY)"
set POWLIB "$env(POW_LIBRARY)/.."
set env(POW_HELPDIR) $env(POW_LIBRARY)

package require Itcl

load [glob $POWLIB/libfitstcl.{s\[ol\]*,dylib}]
load [glob $POWLIB/libpowclient.{s\[ol\]*,dylib}]
# load [glob $POWLIB/libtclxpa.{s\[ol\]*,dylib}]

#if install colormap
set instPos [lsearch $argv "-cmap"]
set colorCode 0
if { $instPos != -1} {
    if { [catch {set colorCode [lindex $argv [expr $instPos+1]]}] == 1} {
        puts $helpMsg
        exit
    }
    if { ($colorCode != 0) && ($colorCode != 1) && ($colorCode != 2) \
        && ($colorCode != 3) } {
        puts $helpMsg
        exit
    }
    set argv [lreplace $argv $instPos [expr $instPos+1]]
    incr argc -2
}


# Look for client/server flag

set instPos [lsearch $argv "-display"]
if { $instPos != -1 } {
   set client [lindex $argv [expr $instPos+1]]
   set argv [lreplace $argv $instPos [expr $instPos+1]]
   incr argc -2
   ::powCmds::remote $client
} else {
   # None specified, so ask POW for default value
   set client [::powCmds::remote]
}


set instPos [lsearch $argv "--"]
if { $instPos != -1 } {
   set argv [lreplace $argv $instPos $instPos]
   incr argc -1
}

::powCmds::init 30 $colorCode

foreach filename $argv {
    regsub -all {[^a-zA-Z0-9.]} [file tail $filename] "_" gname
    set filehandle [fits open $filename 0]
    set imghandle [$filehandle load image]
    set dims [$filehandle info imgdim]
    set n1 [lindex $dims 0]
    if { [llength $dims]==1 } {
       set n2 1
    } else {
       set n2 [lindex $dims 1]
    }
    set data_type [lindex [lindex [$filehandle get keyword BITPIX] 0] 1]
    set data_type $ppFitsDataType($data_type)
    if { ([catch {$filehandle get keyword BZERO}] == 0) ||
         ([catch {$filehandle get keyword BSCALE}] == 0) } {
        set data_type 4
    }

#    powCreateData ${gname}_data $imghandle $data_type [expr $n1 * $n2] 0

    set lstData [list $imghandle $data_type [list $n1 $n2] ]
    ::powCmds::array $lstData ${gname}_data PTR $tcl_platform(byteOrder)

    # Try to get WCS (or scaling) information

    set x0 1
    set y0 1
    set xinc 1
    set yinc 1
    set x_label ""
    set y_label ""
    set x_unit  "pixels"
    set y_unit  "pixels"

    set wcsinfo [getWcs]
    if { [lindex $wcsinfo 4] != "none" } {
       set x_label [lindex [lindex $wcsinfo 3] 0]
       set y_label [lindex [lindex $wcsinfo 3] 1]
       set x_unit "deg"
       set y_unit "deg"
    }

    set result [$filehandle get header2str]
    set cntList(${gname}_img) [getHeaderKeyWord [lindex $result 0] ${gname}_img]
    set powFitsHeaderCnt(${gname}_img) [lindex $cntList(${gname}_img) 1]
    set graphName ${gname}_img
    set powWCSName($graphName) 0
    set powWCSName(${graphName}scope) 0

    if { [lindex $cntList($graphName) 0] > 0 } {
       set powFitsHeader($graphName) [lindex $result 0]
    } else {
       set powFitsHeader($graphName) ""
    }
    
    set currentStr $powFitsHeader($graphName)
    ::powCmds::wcs $graphName $wcsinfo
    ::powCmds::wcsHeader $graphName 0 NONE start

    set idx 0
    set headerLen [string length $currentStr]
    if { $idx + $headerLen <= 3000 } {
       ::powCmds::wcsHeader $graphName $headerLen $currentStr continue
    } else {
       rotationSend [lindex $result 0] $graphName
    }
    ::powCmds::wcsHeader $graphName 0 NONE done

    ::powCmds::wcsHeaderCnt $graphName $powFitsHeaderCnt($graphName)
    ::powCmds::create image $graphName ${gname}_data $n1 $n2

    if ![info exists powWCSList($graphName)] {
       set powWCSList($graphName) {}
       lappend powWCSList($graphName) 1
       lappend powWCSList($graphName) DEFAULT
    }

    foreach name [lindex $powWCSList($graphName) 1] {
       assembleWcsLabel $filehandle $graphName $name
       ::powCmds::wcsLabel $graphName "xlabel" $name $powWCSLabel(xlabel,$graphName,$name)
       ::powCmds::wcsLabel $graphName "ylabel" $name $powWCSLabel(ylabel,$graphName,$name)
       ::powCmds::wcsLabel $graphName "xunit" $name $powWCSLabel(xunit,$graphName,$name)
       ::powCmds::wcsLabel $graphName "yunit" $name $powWCSLabel(yunit,$graphName,$name)
       ::powCmds::wcsLabel $graphName "zlabel" $name $powWCSLabel(zlabel,$graphName,$name)
    }

    set powWCSList(${graphName}scope) $powWCSList($graphName)
    # value
    set wcsliststr "[lindex $powWCSList($graphName) 0]"

    foreach name [lindex $powWCSList($graphName) 1] {
        set wcsliststr [format "%s|%s" $wcsliststr $name]
    }

    ::powCmds::wcsSetList $graphName $wcsliststr

    ::powCmds::graph -name $graphName \
          xlabel $x_label \
          ylabel $y_label \
          xunits $x_unit \
          yunits $y_unit

    ::powCmds::create graph $graphName NULL $graphName

    $filehandle close
}


if { $client != "" } {
   exit
}
